/***************************************************************************
 *   Copyright (C) 2004 by Teemu Rytilahti                                 *
 *   tpr@d5k.net                                                           *
 *                                                                         *
 *   Licensed under GPL.                                                   *
 ***************************************************************************/

#include "konqfeedicon.h"

#include <kapplication.h>
#include <khtml_part.h>
#include <khtmlview.h>
#include <klibloader.h>
#include <kgenericfactory.h>
#include <kparts/statusbarextension.h>

#include <dom/dom_element.h>
#include <dom/dom_doc.h>
#include <dom/dom_string.h>
#include <dom/html_misc.h>

#include <kstandarddirs.h>
#include <dcopclient.h>
#include <kurllabel.h>
#include <kiconloader.h>
#include <kstatusbar.h>
#include <kpopupmenu.h>
#include <kprocess.h>
#include <kmessagebox.h>
#include <kdebug.h>

#include <qtooltip.h>
#include <qcursor.h>
#include <qtimer.h>
#include <qpixmap.h>

typedef KGenericFactory<KonqFeedIcon> KonqFeedIconFactory;
K_EXPORT_COMPONENT_FACTORY(libakregatorkonqfeedicon,
                           KonqFeedIconFactory("akregatorkonqfeedicon"));

KonqFeedIcon::KonqFeedIcon(QObject *parent, const char *name, const QStringList &)
    : KParts::Plugin(parent, name), m_part(0), m_feedIcon(0), m_statusBarEx(0)
{
    KGlobal::locale()->insertCatalogue("akregator_konqplugin");
    
    m_part = static_cast<KHTMLPart*>(parent);
    if(!m_part) { kdDebug() << "couldn't get part" << endl; return; }
    // need to do this because of a bug in khtmlpart
    //connect(m_part->view(), SIGNAL(finishedLayout()), this, SLOT(addFeedIcon()));
    QTimer::singleShot(0, this, SLOT(waitPartToLoad()));
}

void KonqFeedIcon::waitPartToLoad()
{
    connect(m_part, SIGNAL(completed()), this, SLOT(addFeedIcon()));
    connect(m_part, SIGNAL(completed(bool)), this, SLOT(addFeedIcon())); // to make pages with metarefresh to work
    connect(m_part, SIGNAL(started(KIO::Job *)), this, SLOT(removeFeedIcon()));
}

KonqFeedIcon::~KonqFeedIcon()
{
}

// from akregatorplugin.cpp
bool KonqFeedIcon::feedFound()
{
    bool hasFeed = false;
    DOM::Document d=m_part->document();
    DOM::NodeList l=d.getElementsByTagName("link");
    DOM::Element e;
    QString rel;
    QString target;
    for (uint i=0; i < l.length(); i++)
    {
        e=l.item(i);
        if (!e.isNull())
        {
            DOM::DOMString rel=e.getAttribute("rel");
            if (rel=="alternate" || rel=="service.feed")
            {
                DOM::DOMString url=e.getAttribute("href").copy();
                if (!url.isEmpty())
                {
                    m_feedList.append(fixRelativeURL(url.string(), m_part->url()));
                    kdDebug() << "found feed: "<< url.string() << endl;
                    hasFeed=true;
                }
            }
        }
    }

    return hasFeed;
}

void KonqFeedIcon::contextMenu()
{
    KPopupMenu menu;
    menu.insertItem(SmallIcon("bookmark_add"), i18n("Add Feed to aKregator"), this, SLOT(addFeed()));
    menu.exec(QCursor::pos());
}

void KonqFeedIcon::addFeedIcon()
{
    if(feedFound()) {
        if(!m_feedIcon) {
            m_statusBarEx = KParts::StatusBarExtension::childObject(m_part);
            if(!m_statusBarEx) return;
            m_feedIcon = new KURLLabel(m_statusBarEx->statusBar());
            if(!m_feedIcon) return;
            // from khtmlpart's ualabel
            m_feedIcon->setFixedHeight(instance()->iconLoader()->currentSize(KIcon::Small));
            m_feedIcon->setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
            m_feedIcon->setUseCursor(false);
            //FIXME: hackish
            m_feedIcon->setPixmap(QPixmap(locate("data", "akregator/pics/rss.png")));
            
            QToolTip::add(m_feedIcon, i18n("This site has a feed"));
            
            m_statusBarEx->addStatusBarItem(m_feedIcon, 0, true);
        
            connect(m_feedIcon, SIGNAL(leftClickedURL()), this, SLOT(contextMenu()));
        }
    }
}

void KonqFeedIcon::removeFeedIcon()
{
    if(m_feedIcon) {
        m_statusBarEx->removeStatusBarItem(m_feedIcon);
        delete m_feedIcon;
        m_feedIcon = 0;
    }
}

// from akregatorplugin.cpp
void KonqFeedIcon::addFeed()
{
    // Get the DCOP ID of aKregator application.  Any one will do, take first one found.
    DCOPClient *client = kapp->dcopClient();
    QCStringList dcopClients = client->registeredApplications();
    QCStringList::iterator clientIt;
    QCString akregatorClient;
    for (clientIt = dcopClients.begin(); (clientIt != dcopClients.end()) ; ++clientIt )
    {
        if ((*clientIt).contains("akregator", false) != 0) // FIXME: what happens when there's application named akregatorsomething?
        {
            akregatorClient = *clientIt;
            kdDebug() << "KonqFeedIcon::addFeed aKregator DCOP ID is " << akregatorClient << endl;
            break;
        }
    }

    if (akregatorClient.isNull())
    {
        // If aKregator not running, start it using KProcess and pass URLs via command line.
        KProcess *proc = new KProcess;
        *proc << "akregator" << "-g" << i18n("Imported feeds");

        for ( QStringList::Iterator it = m_feedList.begin(); it != m_feedList.end(); ++it ) {
            *proc << "-a" << *it;
        }

        proc->start(KProcess::DontCare);
        delete proc;
    }
    else
    {
        // Locate the object whose name begins with aKregatorPart.  Any one will do, take the first found.
        QCStringList akregatorObjs = client->remoteObjects(akregatorClient);
        QCString akregatorObj;
        for (clientIt = akregatorObjs.begin(); (clientIt != akregatorObjs.end()) ; ++clientIt )
        {
            if ((*clientIt).contains("aKregatorIface", false) != 0)
            {
                akregatorObj = *clientIt;
                kdDebug() << "KonqFeedIcon::addFeed aKregator object is " << akregatorObj << endl;
                break;
            }
        }
        if (akregatorObj.isNull())
            return;

        for ( QStringList::Iterator it = m_feedList.begin(); it != m_feedList.end(); ++it ) {
            kdDebug() << "KonqFeedIcon::addFeed Telling aKregator to add feed " << *it << endl;
            QByteArray  data;
            QDataStream arg(data, IO_WriteOnly);
            arg << *it;
            arg << i18n("Imported feeds");
            if ( !client->send(akregatorClient, akregatorObj, "addFeedToGroup(QString,QString)",
                  data) )
                KMessageBox::error( 0, i18n( "aKregator feed icon - DCOP Call failed" ),
                                      i18n( "The DCOP call addFeedToGroup failed" ));
        }
    }
    m_feedList.clear();
}

QString KonqFeedIcon::fixRelativeURL(const QString &s, const KURL &baseurl)
{
    QString s2=s;
    KURL u;
    if (KURL::isRelativeURL(s2))
    {
        if (s2.startsWith("//"))
        {
            s2=s2.prepend(baseurl.protocol()+":");
            u=s2;
        }
        else if (s2.startsWith("/"))
        {
            u=baseurl;
            u.setPath(s2);
        }
        else
        {
            u=baseurl;
            u.addPath(s2);
        }
        u.cleanPath();
    }
    else
        u=s2;

    u.cleanPath();
    return u.url();
}

#include "konqfeedicon.moc"

